package com.ccwangluo.ccim;


import android.app.Activity;
import android.text.TextUtils;

import com.xuhao.didi.core.pojo.OriginalData;
import com.xuhao.didi.socket.client.sdk.OkSocket;
import com.xuhao.didi.socket.client.sdk.client.ConnectionInfo;
import com.xuhao.didi.socket.client.sdk.client.OkSocketOptions;
import com.xuhao.didi.socket.client.sdk.client.action.SocketActionAdapter;
import com.xuhao.didi.socket.client.sdk.client.connection.IConnectionManager;

import java.util.logging.Logger;

import ccim.protocol.Auth;
import ccim.protocol.Chat;
import ccim.protocol.Common;

public class CCClient {

    private ConnectionInfo authInfo;
    private ConnectionInfo chatInfo;
    private Logger logger = Logger.getLogger(CCClient.class.getName());

    private String token = "";

    private volatile int id = 0;

    private CCClient() {
    }

    private static CCClient mInstance;

    public static CCClient getInstance() {
        if (mInstance == null) {
            synchronized (CCClient.class) {
                if (mInstance == null) {
                    mInstance = new CCClient();
                }
            }
        }
        return mInstance;
    }

    public void init(Activity activity) {
        initAuth();
    }

    private void initChat() {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        chatInfo = new ConnectionInfo("117.144.156.2", 8000);
        //调用OkSocket,开启这次连接的通道,拿到通道Manager
        IConnectionManager manager = OkSocket.open(chatInfo);
        //获得当前连接通道的参配对象
        OkSocketOptions options = manager.getOption();
        //基于当前参配对象构建一个参配建造者类
        OkSocketOptions.Builder builder = new OkSocketOptions.Builder(options);
        //修改参配设置(其他参配请参阅类文档)
        //        builder.setSinglePackageBytes(size);
        //建造一个新的参配对象并且付给通道
        manager.option(builder.build());
        //调用通道进行连接
        manager.connect();

        //设置自定义解析头
        OkSocketOptions.Builder okOptionsBuilder = new OkSocketOptions.Builder();
        okOptionsBuilder.setReaderProtocol(new MyIReaderProtocol());
        //将新的修改后的参配设置给连接管理器
        manager.option(okOptionsBuilder.build());

        manager.registerReceiver(new SocketActionAdapter() {
            @Override
            public void onSocketConnectionSuccess(ConnectionInfo info, String action) {
                super.onSocketConnectionSuccess(info, action);
                logger.info("chat connect success");
            }

            @Override
            public void onSocketReadResponse(ConnectionInfo info, String action, OriginalData data) {
                super.onSocketReadResponse(info, action, data);

                try {
                    Chat.ChatRequestMessage chatRequestMessage = Chat.ChatRequestMessage.parseFrom(data.getBodyBytes());
                    logger.info(String.valueOf(chatRequestMessage));

                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        });
    }

    private void initAuth() {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        authInfo = new ConnectionInfo("117.144.156.2", 9000);
        //调用OkSocket,开启这次连接的通道,拿到通道Manager
        IConnectionManager manager = OkSocket.open(authInfo);
        //获得当前连接通道的参配对象
        OkSocketOptions options = manager.getOption();
        //基于当前参配对象构建一个参配建造者类
        OkSocketOptions.Builder builder = new OkSocketOptions.Builder(options);
        //修改参配设置(其他参配请参阅类文档)
        //        builder.setSinglePackageBytes(size);
        //建造一个新的参配对象并且付给通道
        manager.option(builder.build());
        //调用通道进行连接
        manager.connect();

        //设置自定义解析头
        OkSocketOptions.Builder okOptionsBuilder = new OkSocketOptions.Builder();
        okOptionsBuilder.setReaderProtocol(new MyIReaderProtocol());
        //将新的修改后的参配设置给连接管理器
        manager.option(okOptionsBuilder.build());

        manager.registerReceiver(new SocketActionAdapter() {
            @Override
            public void onSocketConnectionSuccess(ConnectionInfo info, String action) {
                super.onSocketConnectionSuccess(info, action);
                logger.info("auth connect success");

            }

            @Override
            public void onSocketDisconnection(ConnectionInfo info, String action, Exception e) {
                super.onSocketDisconnection(info, action, e);
                logger.info("auth connect success");
            }

            @Override
            public void onSocketReadResponse(ConnectionInfo info, String action, OriginalData data) {
                super.onSocketReadResponse(info, action, data);

                try {
                    Auth.AuthResponseMessage chatResponseMessage = Auth.AuthResponseMessage.parseFrom(data.getBodyBytes());
                    logger.info(String.valueOf(chatResponseMessage));
                    token = chatResponseMessage.getLoginRes().getToken();
                    initChat();
                    manager.disconnect();
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        });
    }


    public void login() {
        Auth.LoginRequest.Builder builder = Auth.LoginRequest.newBuilder().setUsername("user1").setPassword("123456").setAppId(1);
        Auth.AuthRequestMessage build = Auth.AuthRequestMessage.newBuilder().setLoginReq(builder.build()).build();
        OkSocket.open(authInfo)
                .send(new IrequestData(build.toByteArray()));

    }

    public void connectChatServer(){
        if (TextUtils.isEmpty(token)){
            return;
        }
        Chat.ConnectRequest builder = Chat.ConnectRequest.newBuilder().setToken(token).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setConnectReq(builder).setHeader(header).build();
        OkSocket.open(chatInfo)
                .send(new IrequestData(build.toByteArray()));
        id++;
    }


    public void getChatList(){
        if (TextUtils.isEmpty(token)){
            return;
        }
        Chat.PullContactsRequest builder = Chat.PullContactsRequest.newBuilder()
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setPullContactsReq(builder).setHeader(header).build();
        OkSocket.open(chatInfo)
                .send(new IrequestData(build.toByteArray()));
        id++;
    }




}
